#!/bin/sh

# iostat-bf.sh 

# $Id: iostat-bf.sh,v 1.9 2003/03/18 16:48:12 cncook Exp $
#
# Larrd Bottom feeder to check I/O on Solaris
# This version will check Veritas file systems as well as ufs
#
# v1.0 Craig Cook
#   Added patch supplied by John Clark
#   Modified patch to work with and without Veritas
#
# Instructions
# ------------
# 
# Uncomment the VERITAS_CMD line that you require.
# Replace your iostat-bf.sh file with this one.
#
#
#    WARNING: the comments below are just a guide, seek advice from your
#             security team before following these instructions
#             NOTE: this will also allow "bbuser" to change veritas information.
#                   if that is not wanted, you should probably use sudo
#
#               I had problems where "bbuser" could not run vxdisk...
#               to make this work I copied the vxprint binary into
#                 $BBHOME/ext/bin
#                 cd $BBHOME/ext/bin
#                 chown root:bb vxprint
#                 chmod u+s vxprint
#                 chmod g+x vxprint  (allow big brother to execute it as root)
#                 chmod o-a vxprint  (don't allow world group access)
#               change permissions on the dir so only big brother can
#                 get into it (chmod 750 $BBHOME/ext/bin)
#
#             Another method is to use "sudo" - one catch though, your sudo
#               logs will fill up quickly
#             Add something similar to this to your sudo config file
#
#             Cmmd_Alias  BBPROCS = /usr/sbin/vxprint -ht
#             bbuser   ALL=NOPASSWD:BBPROCS

MET=iostat
NETSTAT=/bin/iostat
OUT=/tmp/iostat.$$
VXPRINT_CMD="/usr/sbin/vxprint -ht"
#VXPRINT_CMD="$BBHOME/bin/vxprint -ht"
#VXPRINT_CMD="/usr/local/bin/sudo vxprint -ht"

if [ $BBOSTYPE != "solaris" ]
then
	echo
	echo "iostat has only been tested on Solaris > 2.6 "
	echo
	exit 1
fi

#################
# COLLECT
#################
 
# first stab at dealing with getting disk io
# metrics from solaris 2.6 iostat

KEY=/tmp/key.$$
NEWKEY=/tmp/newkey.$$
LOG=/tmp/iostat.out.$$
LOG2=/tmp/iostat.out2.$$
DISKS=/tmp/disks.$$
FINAL=/tmp/iostat.final.$$


# simple check for Veritas
if [ -f /usr/sbin/vxprint ]; then
  HAVE_VERITAS=Y
else
  HAVE_VERITAS=N
fi


# create a key for filesystems we care about
# the ufs greps
if [ "$HAVE_VERITAS" = "Y" ]; then
  $VXPRINT_CMD | grep '^sd' | awk '{print$8,$3}' > $KEY
else
  /usr/sbin/mount -p | $GREP " ufs " | $GREP -v " ro" | awk '{print $1,$3}' > $KEY
fi


# now let's create a list of the goods
if [ -f $DISKS ]
then
        rm $DISKS
fi

if [ -f $NEWKEY ]
then
        rm $NEWKEY
fi


cat $KEY | {
while read line
do
        set $line
        BASE=`basename $1`
        echo $BASE >> $DISKS
        echo $BASE $2 >> $NEWKEY
done
}


# go ahead and get the iostat
# To make this more accurate should take a larger sample and then get
# the average
if [ "$HAVE_VERITAS" = "Y" ]; then
  iostat -xnpe 100 2 > $LOG
else
  iostat -xnpe 100 2 | $SED 's/ md/ d/' > $LOG
fi

# now get rid of the first half of data
LINES=`cat $LOG | wc -l `
HALF=`expr $LINES / 2`
tail -$HALF $LOG > $LOG2

if [ -f $FINAL ]
then
        rm $FINAL
fi

# now parse the $LOG2 for what we want
for disk in `cat $DISKS`
do
        $GREP " $disk$" $LOG2 >> $FINAL
done

# now lets print what we will send to BB
echo "BEGINKEY" > $OUT
cat $NEWKEY >> $OUT
echo "ENDKEY" >> $OUT

echo "BEGINDATA" >> $OUT
sed -n '2p' $LOG2 >> $OUT
cat $FINAL >> $OUT
echo "ENDDATA" >> $OUT

#################
# REPORT
#################
if [ $LARRDCOMM = DATA ]
then
        $BB $BBDISP "data ${MACHINE}.$MET
`$CAT $OUT`"

else
        $BB $BBDISP "status ${MACHINE}.$MET green `date`  <$MET>
`$CAT $OUT`"

fi

rm $OUT $DISKS $KEY $NEWKEY $FINAL $LOG $LOG2
